"""Unit test package for xpcsviewer."""

from xpcsviewer.utils.logging_config import get_logger

# Initialize test package logger
logger = get_logger(__name__)
logger.info("XPCS Viewer test package initialized")
